<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class Estrutura extends \Zeedhi\Framework\Controller\Simple {
    
    protected $salesMessage;
    protected $consumidorService;
    
    public function __construct(Services\Estrutura $estruturaService, Utilities\SalesMessage $salesMessage, Utilities\Util $util) {
        $this->estruturaService = $estruturaService;
        $this->salesMessage = $salesMessage;
        $this->util = $util;
    }	      

    public function insereTipoEstrutura(Request\Filter $request, Response $response) {
    	try{
    		//-----------Prepara dados para a inserção
    		$params = $request->getFilterCriteria()->getConditions();
            $NMTIPOESTRUTURA = $params[0]['value'];
            $NRTIPOESTRUTURA = $this->util->newCode('TIPOESTRUTURA', '12');
            $NRORG = '1';    //----Organizaçao padrão
            //---- chama service de inserção
            $this->estruturaService->insereTipoEstrutura($NRTIPOESTRUTURA, $NMTIPOESTRUTURA, $NRORG);
    		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('AdicionaTipoDeEstruturaRepo', array('mensagemSucesso' => $this->salesMessage->getMessage(66))));	
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

    public function insereEstrutura(Request\Filter $request, Response $response) {
    	try{
    		//-----------Prepara dados para a inserção
    		$params = $request->getFilterCriteria()->getConditions();
            $NRORG = '1';
            $NRTIPOESTRUTURA = $params[0]['value'];
            $NMESTRUTURAM = $params[1]['value'];
            $NRESTRUTURAM = $this->util->newCode('ESTRUTURAM', '12');
            $CDINTESTRUTURA = $params[2]['value'];
            $NRESTRUTURAMSUP = $params[3]['value'];
            $NRTIPOHIERARQORG = $params[4]['value'];

            if($NRESTRUTURAMSUP != 'nulo') {
            	$NRHIERARQUIAORG = $this->util->newCode('NRHIERARQUIAORG', '12');
            	$this->estruturaService->insereHierarquia($NRHIERARQUIAORG, $NRORG, $NRTIPOHIERARQORG, $NRESTRUTURAMSUP, $NRESTRUTURAM);
            }
            
            //---- chama service de inserção
            $this->estruturaService->insereEstrutura($NRESTRUTURAM, $NRORG, $NRTIPOESTRUTURA, $NMESTRUTURAM, $CDINTESTRUTURA);
    		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('AdicionaEstruturaRepo', array('mensagemSucesso' => $this->salesMessage->getMessage(67))));
    		$estruturasHieraquizadas = $this->estruturaService->buscaEstruturasHieraquizadas();
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaEstruturasHieraquizadasRepo', $estruturasHieraquizadas));
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

   	public function buscaEstruturas(Request\Filter $request, Response $response) {
    	try{
    		$params = $request->getFilterCriteria()->getConditions();
    		$NRTIPOESTRUTSUP = $params[0]['value'];   		
    	    $estruturas = $this->estruturaService->buscaEstruturas($NRTIPOESTRUTSUP);
    		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('EstruturasRepo', $estruturas));
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

    public function buscaTiposHierarquia(Request\Filter $request, Response $response) {
    	try{
    		$params = $request->getFilterCriteria()->getConditions();
    		if($params == array()){
    			$params[0]['value']='';
    		}
    		$NRTIPOESTRUTREL = $params[0]['value'];
            if($NRTIPOESTRUTREL!= ""){
	            $tiposHierarquia = $this->estruturaService->buscaTiposHierarquia($NRTIPOESTRUTREL);
            } else{
            	$tiposHierarquia = $this->estruturaService->buscaTiposHierarquiaSemFiltro();
            }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TiposDeHierarquiaRepo', $tiposHierarquia));
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

    public function insereTipoHierarquia(Request\Filter $request, Response $response) {
    	try{
    		//-----------Prepara dados para a inserção
    		$params = $request->getFilterCriteria()->getConditions();
    		$NMTIPOHIERARQORG = $params[0]['value'];
    		$NRTIPOESTRUTREL = $params[1]['value'];
    		$NRTIPOESTRUTSUP = $params[2]['value'];

            $NRORG = '1';  //----Organizaçao padrão
            $NRTIPOHIERARQORG = $this->util->newCode('TIPOHIERARQUIA', '12');
            //---- chama service de inserção
            
            $this->estruturaService->insereTipoHierarquia($NRTIPOHIERARQORG, $NMTIPOHIERARQORG, $NRTIPOESTRUTREL, $NRTIPOESTRUTSUP, $NRORG);
            $tiposHierarquia = $this->estruturaService->buscaTiposHierarquia($NRTIPOESTRUTREL);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TiposDeHierarquiaRepo', $tiposHierarquia));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('AdicionaTipoHierarquiaRepo', array('mensagemSucesso' => $this->salesMessage->getMessage(68))));
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

    public function buscaTiposEstrutura(Request\Filter $request, Response $response) {
    	try{	
            $tiposDeEstrutura = $this->estruturaService->buscaTiposEstrutura();
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('TiposDeEstruturaRepo', $tiposDeEstrutura));
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

    public function buscaEstruturasHieraquizadas(Request\Filter $request, Response $response) {
    	try{	
            $estruturasHieraquizadas = $this->estruturaService->buscaEstruturasHieraquizadas();
            $idxPai = array();
        	foreach ($estruturasHieraquizadas as $estrutura){
        		$idxPai[$estrutura['NRESTRUTURAM']] = $estrutura['NMESTRUTURAM'];
        	}

            foreach ($estruturasHieraquizadas as &$estrutura) {
            	if(empty($estrutura['NRESTRUTURAMSUP'])){
            		$estrutura['NMPAI'] = null;
            	} else{
            		$estrutura['NMPAI'] = $idxPai[$estrutura['NRESTRUTURAMSUP']];
            	}
            }
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaEstruturasHieraquizadasRepo', $estruturasHieraquizadas));
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

    
    public function excluiEstrutura(Request\Filter $request, Response $response) {
    	try{
    		$params = $request->getFilterCriteria()->getConditions();
    		$NRESTRUTURAM = $params[0]['value'];
    		$NRESTRUTURAMSUP = $params[1]['value'];
    		if($NRESTRUTURAMSUP=='nulo'){
    			$NRESTRUTURAMSUP = null;
    		} 
    		//----busca operadores associados a esta estrutura
    		$operadoresAssociados = $this->estruturaService->buscaOperAssociados($NRESTRUTURAM);
    		if(empty($operadoresAssociados)) {
	    		//----busca filhos da estrutura que será excluída
	    		$filhos = $this->estruturaService->buscaFilhosEstrutura($NRESTRUTURAM);
	    		//----troca o pai dos filhos da estrutrutura que sera exlcuída pelo seu avô
	    		if(!empty($filhos)){
	    			foreach ($filhos as $filho) {
	    				//troca hierarquia de pai->filho (Coloca o vô como pai)
	    				$this->estruturaService->refazHierarquia($filho['NRESTRUTURAMREL'], $NRESTRUTURAMSUP);
	    			}
	    		}
	    		$this->estruturaService->excluiHierarquia($NRESTRUTURAM);
	    		$this->estruturaService->excluiEstrutura($NRESTRUTURAM);

	            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('DeletaEstruturaRepo', array('mensagemSucesso' => $this->salesMessage->getMessage(69))));
	            $estruturasHieraquizadas = $this->estruturaService->buscaEstruturasHieraquizadas();
	            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaEstruturasHieraquizadasRepo', $estruturasHieraquizadas));
	        } else {
	        	$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('DeletaEstruturaRepo', array('mensagemSucesso' => $this->salesMessage->getMessage(79))));
	            $estruturasHieraquizadas = $this->estruturaService->buscaEstruturasHieraquizadas();
	            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaEstruturasHieraquizadasRepo', $estruturasHieraquizadas));
	        }
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }


    public function alteraEstrutura(Request\Filter $request, Response $response) {
    	try{
    		$params = $request->getFilterCriteria()->getConditions();
    		$NRESTRUTURAM = $params[0]['value'];
    		$NRESTRUTURAMSUP = $params[1]['value'];
    		$NMESTRUTURAM = $params[2]['value'];
    		$NRTIPOHIERARQORG = $params[3]['value'];
    		$NRORG = '1';
    		if(empty($NRESTRUTURAMSUP) && empty($NRTIPOHIERARQORG)){
    			//--altera o nome da estrutura
    			$this->estruturaService->alteraNomeEstrutura($NRESTRUTURAM, $NMESTRUTURAM);
    		} else{
    			//--altera o nome da estrutura
	    		$this->estruturaService->alteraNomeEstrutura($NRESTRUTURAM, $NMESTRUTURAM);
	    		//--altera a hieraquia
	    		$linhasAlteradas = $this->estruturaService->alteraEstruturaPai($NRESTRUTURAM, $NRESTRUTURAMSUP, $NRTIPOHIERARQORG);
	    		if($linhasAlteradas->rowCount() == 0){
	    			$NRHIERARQUIAORG = $this->util->newCode('NRHIERARQUIAORG', '12');
	    			$this->estruturaService->insereHierarquia($NRHIERARQUIAORG, $NRORG, $NRTIPOHIERARQORG, $NRESTRUTURAMSUP, $NRESTRUTURAM);
	    		}
    		}
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('AlteraEstruturaRepo', array('mensagemSucesso' => $this->salesMessage->getMessage(70))));
            $estruturasHieraquizadas = $this->estruturaService->buscaEstruturasHieraquizadas();
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaEstruturasHieraquizadasRepo', $estruturasHieraquizadas));
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

    public function buscaOperadoresAssociadosEstrutura(Request\Filter $request, Response $response) {
    try{
    		$params = $request->getFilterCriteria()->getConditions();
    		$NRESTRUTURAM = $params[0]['value'];
    		$retorno = self::buscaOperadoresdaEstrutura($NRESTRUTURAM);
    		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaOperadoresAssociadosEstruturaRepo', $retorno));
       	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

    private function buscaOperadoresdaEstrutura($NRESTRUTURAM) {
    		$NRORG = '1';
    		$estruturasOperador = $this->estruturaService->buscaEstruturaOperador($NRESTRUTURAM);
    		$operadores = array();
    		$organizacaoOperador = array();

    		foreach ($estruturasOperador as $estrutop) {
    			$org = $this->estruturaService->buscaOrganizacaoOperador($estrutop['NRORGANIZACAOOPER'], $NRORG);
    			array_push($organizacaoOperador, $org);
    		
    		}
    		foreach ($organizacaoOperador as $orgoperador) {
    			$oper = $this->estruturaService->buscaOperadorOrganizacao($orgoperador[0]['CDOPERADOR']);
    			array_push($operadores, $oper[0]);
    		}
    		return $operadores;
    }


    public function buscaOperadoresAssociar(Request\Filter $request, Response $response) {
    	try{
    		$params = $request->getFilterCriteria()->getConditions();
            if(isset($params[0])){
            	$NMOPERADOR = $params[0]['value'];
            }else{
            	$NMOPERADOR = "%%";
            }
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $FIRST = $request->getFilterCriteria()->getPageSize() * ($page-1);
            $LAST = ($FIRST + $pageSize);
            if($page > 1){
            	$FIRST++;
            }
            $operadores = $this->estruturaService->buscaOperadoresAssociar($NMOPERADOR, $FIRST, $LAST);
    		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('OperadoresAssociarRepo', $operadores));
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }


    public function associaOperadorEstrutura(Request\Filter $request, Response $response) {
    	try{
    		$params = $request->getFilterCriteria()->getConditions();
    		$CDOPERADOR = $params[0]['value'];
    		$NRESTRUTURAM = $params[1]['value'];
    		$NRORG = '1';
    		$orgOper = $this->estruturaService->buscaOrgOperador($CDOPERADOR, $NRORG);

    		if($orgOper == array()){
    			$NRORGANIZACAOOPER = $this->util->newCode('ORGANIZACAOOPER', '12');
    			$this->estruturaService->insertOrganizacaoOperador($CDOPERADOR, $NRORGANIZACAOOPER , $NRORG);
    		} else{
    			$NRORGANIZACAOOPER = $orgOper[0]['NRORGANIZACAOOPER'];
    		}
    		$NRESTRUTURAOPER = $this->util->newCode('ESTRUTURAOPER', '12');
    		$retorno = self::buscaOperadoresdaEstrutura($NRESTRUTURAM);
    		foreach ($retorno as $operador) {
    			if($operador['CDOPERADOR']==$CDOPERADOR){
    				throw new \Exception ($this->salesMessage->getMessage(75));
    			}
    		}
    		$this->estruturaService->insertEstruturaOperador($NRESTRUTURAOPER, $NRORGANIZACAOOPER , $NRESTRUTURAM);	
            $retorno = self::buscaOperadoresdaEstrutura($NRESTRUTURAM);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaOperadoresAssociadosEstruturaRepo', $retorno));
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('AssociaOperadorEstruturaRepo', array('mensagem' => $this->salesMessage->getMessage(74))));
    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

    public function excluiAssociacaoOperador(Request\Filter $request, Response $response) {
    	try{
    		$params = $request->getFilterCriteria()->getConditions();
    		$CDOPERADOR = $params[0]['value'];
    		$NRESTRUTURAM = $params[1]['value'];
    		$NRORG = '1';
    		$orgOper = $this->estruturaService->buscaOrgOperador($CDOPERADOR, $NRORG);
    		$NRORGANIZACAOOPER = $orgOper[0]['NRORGANIZACAOOPER'];
    		$this->estruturaService->excluiAssociacaoOperador($NRESTRUTURAM, $NRORGANIZACAOOPER);
    		$retorno = self::buscaOperadoresdaEstrutura($NRESTRUTURAM);
    		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('BuscaOperadoresAssociadosEstruturaRepo', $retorno));		
    		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('ExcluiAssociacaoOperadorEstruturaRepo', array('mensagem' => $this->salesMessage->getMessage(78))));

    	} catch (\Exception $e) {
    		$response->addMessage(new Message($e->getMessage()));
    	}
    }

}